#include "pins_arduino.h"

// Константы для I2C общения к ведомому контроллеру V2R
#include "v2r_i2c_types.h"

// Работа с I2C в качестве Slave устройства
#include <TinyWireS.h>

// Работа с сервоприводами
#include <SoftwareServo.h>

// Статус устройства на момент выполнения аоследней команды
byte lastCommandStatus = V2R_I2C_STATUS_NONE;

// Создаём объекты для работы с сервами
SoftwareServo servo_1;
SoftwareServo servo_2;
SoftwareServo servo_3;

/*
  Инициализация приложения
*/
void setup ()
{
  delay(100); // give the master some time to grab the i2c bus
  sei(); // enable interrupts

  // Инициализация I2C в качестве ведомого устройства на стантадрный адрес
  TinyWireS.begin(I2C_SLAVE_V2R_I2C);

  // Подрубаем пины для работы с сервами к объектам по управлению сервами
  // Это пины PB1, PB3 и PB4
  // Стот отметить, что ШИМ для работы с сервами не "родной", а программный))
  // На ATtiny45 можно все пины использовать под сервы, кроме PB5 - глючит
  servo_1.attach(1);
  servo_2.attach(3);
  servo_3.attach(4);

  // Устанавливаем сервы в дефолтное положение
  servo_1.write(90);
  servo_2.write(90);
  servo_3.write(90);
  SoftwareServo::refresh();
}

/*
  Цикл приложения
*/
void loop ()
{
  // Поддерживаем сервы в установленном положении
  servo_1.write(servo_1.read());
  servo_2.write(servo_2.read());
  servo_3.write(servo_3.read());

  // Ожидаем сообщения по I2C
  if (TinyWireS.available())
  {
    // Читаем данные с I32C шины
    byte i2cData = TinyWireS.receive();

    switch (i2cData)
    {
      // Устанавливает положение сервы 1
      case V2R_I2C_SERVO_1_SET:
        // Получаем значение положения сервы
        i2cData = TinyWireS.receive();

        // Устанавливаем положение сервы
        servo_1.write(i2cData);

        // Ставим статус устройства
        lastCommandStatus = V2R_I2C_STATUS_SERVO_1_SET;
      break;

      // Устанавливает положение сервы 2
      case V2R_I2C_SERVO_2_SET:
        // Получаем значение положения сервы
        i2cData = TinyWireS.receive();

        // Устанавливаем положение сервы
        servo_2.write(i2cData);

        // Ставим статус устройства
        lastCommandStatus = V2R_I2C_STATUS_SERVO_2_SET;
      break;

      // Устанавливает положение сервы 3
      case V2R_I2C_SERVO_3_SET:
        // Получаем значение положения сервы
        i2cData = TinyWireS.receive();

        // Устанавливаем положение сервы
        servo_3.write(i2cData);

        // Ставим статус устройства
        lastCommandStatus = V2R_I2C_STATUS_SERVO_3_SET;
      break;

      // Узнаёт положение сервы 1
      case V2R_I2C_SERVO_1_GET:
        // Возвращаем данные I2C мастеру
        TinyWireS.send((byte)servo_1.read());

        return;
      break;

      // Узнаёт положение сервы 2
      case V2R_I2C_SERVO_2_GET:
        // Возвращаем данные I2C мастеру
        TinyWireS.send((byte)servo_2.read());

        return;
      break;

      // Узнаёт положение сервы 3
      case V2R_I2C_SERVO_3_GET:
        // Возвращаем данные I2C мастеру
        TinyWireS.send((byte)servo_3.read());

        return;
      break;

      // Смена идентификатора контроллера для в I2C шине
      case V2R_I2C_CHANGE_I2C_ID:
        // Получаем новый идентификатор устройства
        i2cData = TinyWireS.receive();

        // Переинициализация I2C в качестве ведомого устройства на новый адрес
        TinyWireS.begin(i2cData);

        // Ставим статус устройства
        lastCommandStatus = V2R_I2C_STATUS_CHANGE_I2C_ID;
      break;

      // Сброс всех установок
      case V2R_I2C_RESET:
        // Устанавливаем сервы в дефолтное положение
        servo_1.write(90);
        servo_2.write(90);
        servo_3.write(90);

        // Ставим статус устройства
        lastCommandStatus = V2R_I2C_STATUS_NONE;
      break;

      // Запрос состояния контроллера
      case V2R_I2C_GET_STATUS:
        // Возвращаем данные I2C мастеру
        TinyWireS.send((byte)lastCommandStatus);

        return;
      break;
    }

    // Возвращаем данные I2C мастеру
    TinyWireS.send(0);
  }

  SoftwareServo::refresh();
}
